# not an eol comment
_SITES_extract:=        ${_SITES_extract} 69${GH_SUBDIR_${_group}:C=[^/]+= =g:[#]}:post-extract-gh-${_group}
<<<<<<<<<
# not an eol comment
_SITES_extract:=	${_SITES_extract} \
			69${GH_SUBDIR_${_group}:C=[^/]+= =g:[#]}:post-extract-gh-${_group}
