${PORTEDIT} bump-revision $input | diff -L $expected -L $actual -u $expected -
<<<<<<<<<
PORTNAME=			ghostwriter
DISTVERSIONPREFIX=	v
DISTVERSION=			1.7.4
DISTVERSIONSUFFIX=	-foo
CATEGORIES=	editors
<<<<<<<<<
PORTNAME=			ghostwriter
DISTVERSIONPREFIX=	v
DISTVERSION=			1.7.4
DISTVERSIONSUFFIX=	-foo
PORTREVISION=	1
CATEGORIES=	editors
