SUBPACKAGES=gdbm
COMMENT.gdbm=foo
COMMENT=fo

.include <bsd.port.pre.mk>
.include <bsd.port.post.mk>
<<<<<<<<<
# Maintainer block
+COMMENT
+COMMENT.gdbm

# Subpackages block
SUBPACKAGES

-COMMENT.gdbm
-COMMENT
