${PORTEDIT} bump-revision $input | diff -L $expected -L $actual -u $expected -
<<<<<<<<<
PORTNAME=	lastpass-cli
PORTVERSION=	1.3.1
DISTVERSIONPREFIX=	v
CATEGORIES=	security
<<<<<<<<<
PORTNAME=	lastpass-cli
PORTVERSION=	1.3.1
DISTVERSIONPREFIX=	v
PORTREVISION=	1
CATEGORIES=	security
