SHEBANG_FILES=	utils/lit/lit.py utils/llvm-lit/llvm-lit.in \
		tools/clang/tools/clang-format/clang-format-diff.py \
		tools/clang/tools/extra/clang-tidy/tool/clang-tidy-diff.py \
		tools/clang/tools/extra/clang-tidy/tool/run-clang-tidy.py \
		tools/clang/tools/extra/include-fixer/find-all-symbols/tool/run-find-all-symbols.py \
		tools/clang/utils/hmaptool/hmaptool \
		tools/opt-viewer/optrecord.py \
		tools/opt-viewer/opt-diff.py \
		tools/opt-viewer/opt-stats.py \
		tools/opt-viewer/opt-viewer.py
<<<<<<<<<
SHEBANG_FILES=	tools/clang/tools/clang-format/clang-format-diff.py \
		tools/clang/tools/extra/clang-tidy/tool/clang-tidy-diff.py \
		tools/clang/tools/extra/clang-tidy/tool/run-clang-tidy.py \
		tools/clang/tools/extra/include-fixer/find-all-symbols/tool/run-find-all-symbols.py \
		tools/clang/utils/hmaptool/hmaptool \
		tools/opt-viewer/opt-diff.py tools/opt-viewer/opt-stats.py \
		tools/opt-viewer/opt-viewer.py tools/opt-viewer/optrecord.py \
		utils/lit/lit.py utils/llvm-lit/llvm-lit.in
