/* Generated by generate_rules.sh; do not edit */
#include "config.h"
#include <sys/types.h>
#include <inttypes.h>
#include "constants.h"
struct ConstantStaticFlavors static_flavors[] = {
	{ "emacs", "canna" },
	{ "emacs", "devel_full" },
	{ "emacs", "devel_nox" },
	{ "emacs", "full" },
	{ "emacs", "nox" },
	{ "lazarus", "gtk2" },
	{ "lazarus", "qt5" },
	{ "lua", "lua51" },
	{ "lua", "lua52" },
	{ "lua", "lua53" },
	{ "lua", "lua54" },
	{ "php", "php74" },
	{ "php", "php80" },
	{ "php", "php81" },
	{ "python", "py27" },
	{ "python", "py310" },
	{ "python", "py311" },
	{ "python", "py37" },
	{ "python", "py38" },
	{ "python", "py39" },
};
size_t static_flavors_len = nitems(static_flavors);
const char *known_architectures[] = {
	"aarch64",
	"amd64",
	"arm",
	"armv6",
	"armv7",
	"i386",
	"mips",
	"mips64",
	"mips64el",
	"mips64elhf",
	"mips64hf",
	"mipsel",
	"mipselhf",
	"mipshf",
	"mipsn32",
	"powerpc",
	"powerpc64",
	"powerpc64le",
	"powerpcspe",
	"riscv64",
	"riscv64sf",
	"sparc64",
};
size_t known_architectures_len = nitems(known_architectures);
uint32_t freebsd_versions[] = {
	11,
	12,
	13,
	14,
};
size_t freebsd_versions_len = nitems(freebsd_versions);
const char *use_gnome_rel[] = {
	"atk",
	"atkmm",
	"cairo",
	"cairomm",
	"dconf",
	"evolutiondataserver3",
	"gconf2",
	"gconfmm26",
	"gdkpixbuf2",
	"glib20",
	"glibmm",
	"gnomecontrolcenter3",
	"gnomedesktop3",
	"gnomemenus3",
	"gnomemimedata",
	"gnomeprefix",
	"gsound",
	"gtk-update-icon-cache",
	"gtk20",
	"gtk30",
	"gtk40",
	"gtkhtml4",
	"gtkmm24",
	"gtkmm30",
	"gtksharp20",
	"gtksourceview2",
	"gtksourceview3",
	"gtksourceview4",
	"gtksourceview5",
	"gtksourceviewmm3",
	"gvfs",
	"intlhack",
	"intltool",
	"introspection",
	"libadwaita",
	"libartlgpl2",
	"libgda5",
	"libgda5-ui",
	"libgdamm5",
	"libglade2",
	"libgnomecanvas",
	"libgnomekbd",
	"libgsf",
	"libidl",
	"librsvg2",
	"libsigc++20",
	"libwnck",
	"libwnck3",
	"libxml++26",
	"libxml2",
	"libxslt",
	"metacity",
	"nautilus3",
	"pango",
	"pangomm",
	"pangox-compat",
	"pygobject3",
	"vte",
	"vte3",
	"dconf:build",
	"gnomecontrolcenter3:build",
	"gnomemenus3:build",
	"gnomemimedata:build",
	"gsound:build",
	"gtksharp20:build",
	"gvfs:build",
	"intltool:build",
	"introspection:build",
	"librsvg2:build",
	"libxml2:build",
	"libxslt:build",
	"pygobject3:build",
	"dconf:run",
	"gnomecontrolcenter3:run",
	"gnomemenus3:run",
	"gnomemimedata:run",
	"gsound:run",
	"gtk-update-icon-cache:run",
	"gtksharp20:run",
	"gvfs:run",
	"introspection:run",
	"librsvg2:run",
	"libxml2:run",
	"libxslt:run",
	"pygobject3:run",
};
size_t use_gnome_rel_len = nitems(use_gnome_rel);
const char *use_kde_rel[] = {
	"activities",
	"activities-stats",
	"activitymanagerd",
	"akonadi",
	"akonadicalendar",
	"akonadiconsole",
	"akonadicontacts",
	"akonadiimportwizard",
	"akonadimime",
	"akonadinotes",
	"akonadisearch",
	"akregator",
	"alarmcalendar",
	"apidox",
	"archive",
	"attica",
	"attica5",
	"auth",
	"baloo",
	"baloo-widgets",
	"baloo5",
	"bookmarks",
	"breeze",
	"breeze-gtk",
	"breeze-icons",
	"calendarcore",
	"calendarsupport",
	"calendarutils",
	"codecs",
	"completion",
	"config",
	"configwidgets",
	"contacts",
	"coreaddons",
	"crash",
	"dbusaddons",
	"decoration",
	"designerplugin",
	"discover",
	"dnssd",
	"doctools",
	"drkonqi",
	"ecm",
	"emoticons",
	"eventviews",
	"filemetadata",
	"frameworkintegration",
	"gapi",
	"globalaccel",
	"grantlee-editor",
	"grantleetheme",
	"gravatar",
	"guiaddons",
	"holidays",
	"hotkeys",
	"i18n",
	"iconthemes",
	"identitymanagement",
	"idletime",
	"imap",
	"incidenceeditor",
	"infocenter",
	"init",
	"itemmodels",
	"itemviews",
	"jobwidgets",
	"js",
	"jsembed",
	"kaddressbook",
	"kalarm",
	"kate",
	"kcmutils",
	"kdav",
	"kde-cli-tools",
	"kde-gtk-config",
	"kdeclarative",
	"kded",
	"kdelibs4support",
	"kdepim-addons",
	"kdepim-runtime5",
	"kdeplasma-addons",
	"kdesu",
	"kdewebkit",
	"kgamma5",
	"khtml",
	"kimageformats",
	"kio",
	"kirigami2",
	"kitinerary",
	"kmail",
	"kmail-account-wizard",
	"kmenuedit",
	"knotes",
	"kontact",
	"kontactinterface",
	"korganizer",
	"kosm",
	"kpkpass",
	"kpublictransport",
	"kquickcharts",
	"kross",
	"kscreen",
	"kscreenlocker",
	"ksmtp",
	"ksshaskpass",
	"ksysguard",
	"ksystemstats",
	"kwallet-pam",
	"kwayland-integration",
	"kwayland-server",
	"kwin",
	"kwrited",
	"layer-shell-qt",
	"ldap",
	"libkcddb",
	"libkcompactdisc",
	"libkdcraw",
	"libkdegames",
	"libkdepim",
	"libkeduvocdocument",
	"libkexiv2",
	"libkipi",
	"libkleo",
	"libksane",
	"libkscreen",
	"libksieve",
	"libksysguard",
	"mailcommon",
	"mailimporter",
	"mailtransport",
	"marble",
	"mbox",
	"mbox-importer",
	"mediaplayer",
	"messagelib",
	"milou",
	"mime",
	"newstuff",
	"notifications",
	"notifyconfig",
	"okular",
	"oxygen",
	"oxygen-icons5",
	"package",
	"parts",
	"people",
	"pim-data-exporter",
	"pimcommon",
	"pimtextedit",
	"plasma-browser-integration",
	"plasma-desktop",
	"plasma-disks",
	"plasma-framework",
	"plasma-integration",
	"plasma-pa",
	"plasma-sdk",
	"plasma-wayland-protocols",
	"plasma-workspace",
	"plasma-workspace-wallpapers",
	"plotting",
	"polkit-kde-agent-1",
	"powerdevil",
	"prison",
	"pty",
	"purpose",
	"qqc2-desktop-style",
	"runner",
	"service",
	"solid",
	"sonnet",
	"syndication",
	"syntaxhighlighting",
	"systemsettings",
	"texteditor",
	"textwidgets",
	"threadweaver",
	"tnef",
	"unitconversion",
	"wallet",
	"wayland",
	"widgetsaddons",
	"windowsystem",
	"xmlgui",
	"xmlrpcclient",
	"activities_build",
	"activities-stats_build",
	"activitymanagerd_build",
	"akonadi_build",
	"akonadicalendar_build",
	"akonadiconsole_build",
	"akonadicontacts_build",
	"akonadiimportwizard_build",
	"akonadimime_build",
	"akonadinotes_build",
	"akonadisearch_build",
	"akregator_build",
	"alarmcalendar_build",
	"apidox_build",
	"archive_build",
	"attica_build",
	"attica5_build",
	"auth_build",
	"baloo_build",
	"baloo-widgets_build",
	"baloo5_build",
	"bookmarks_build",
	"breeze_build",
	"breeze-gtk_build",
	"breeze-icons_build",
	"calendarcore_build",
	"calendarsupport_build",
	"calendarutils_build",
	"codecs_build",
	"completion_build",
	"config_build",
	"configwidgets_build",
	"contacts_build",
	"coreaddons_build",
	"crash_build",
	"dbusaddons_build",
	"decoration_build",
	"designerplugin_build",
	"discover_build",
	"dnssd_build",
	"doctools_build",
	"drkonqi_build",
	"ecm_build",
	"emoticons_build",
	"eventviews_build",
	"filemetadata_build",
	"frameworkintegration_build",
	"gapi_build",
	"globalaccel_build",
	"grantlee-editor_build",
	"grantleetheme_build",
	"gravatar_build",
	"guiaddons_build",
	"holidays_build",
	"hotkeys_build",
	"i18n_build",
	"iconthemes_build",
	"identitymanagement_build",
	"idletime_build",
	"imap_build",
	"incidenceeditor_build",
	"infocenter_build",
	"init_build",
	"itemmodels_build",
	"itemviews_build",
	"jobwidgets_build",
	"js_build",
	"jsembed_build",
	"kaddressbook_build",
	"kalarm_build",
	"kate_build",
	"kcmutils_build",
	"kdav_build",
	"kde-cli-tools_build",
	"kde-gtk-config_build",
	"kdeclarative_build",
	"kded_build",
	"kdelibs4support_build",
	"kdepim-addons_build",
	"kdepim-runtime5_build",
	"kdeplasma-addons_build",
	"kdesu_build",
	"kdewebkit_build",
	"kgamma5_build",
	"khtml_build",
	"kimageformats_build",
	"kio_build",
	"kirigami2_build",
	"kitinerary_build",
	"kmail_build",
	"kmail-account-wizard_build",
	"kmenuedit_build",
	"knotes_build",
	"kontact_build",
	"kontactinterface_build",
	"korganizer_build",
	"kosm_build",
	"kpkpass_build",
	"kpublictransport_build",
	"kquickcharts_build",
	"kross_build",
	"kscreen_build",
	"kscreenlocker_build",
	"ksmtp_build",
	"ksshaskpass_build",
	"ksysguard_build",
	"ksystemstats_build",
	"kwallet-pam_build",
	"kwayland-integration_build",
	"kwayland-server_build",
	"kwin_build",
	"kwrited_build",
	"layer-shell-qt_build",
	"ldap_build",
	"libkcddb_build",
	"libkcompactdisc_build",
	"libkdcraw_build",
	"libkdegames_build",
	"libkdepim_build",
	"libkeduvocdocument_build",
	"libkexiv2_build",
	"libkipi_build",
	"libkleo_build",
	"libksane_build",
	"libkscreen_build",
	"libksieve_build",
	"libksysguard_build",
	"mailcommon_build",
	"mailimporter_build",
	"mailtransport_build",
	"marble_build",
	"mbox_build",
	"mbox-importer_build",
	"mediaplayer_build",
	"messagelib_build",
	"milou_build",
	"mime_build",
	"newstuff_build",
	"notifications_build",
	"notifyconfig_build",
	"okular_build",
	"oxygen_build",
	"oxygen-icons5_build",
	"package_build",
	"parts_build",
	"people_build",
	"pim-data-exporter_build",
	"pimcommon_build",
	"pimtextedit_build",
	"plasma-browser-integration_build",
	"plasma-desktop_build",
	"plasma-disks_build",
	"plasma-framework_build",
	"plasma-integration_build",
	"plasma-pa_build",
	"plasma-sdk_build",
	"plasma-wayland-protocols_build",
	"plasma-workspace_build",
	"plasma-workspace-wallpapers_build",
	"plotting_build",
	"polkit-kde-agent-1_build",
	"powerdevil_build",
	"prison_build",
	"pty_build",
	"purpose_build",
	"qqc2-desktop-style_build",
	"runner_build",
	"service_build",
	"solid_build",
	"sonnet_build",
	"syndication_build",
	"syntaxhighlighting_build",
	"systemsettings_build",
	"texteditor_build",
	"textwidgets_build",
	"threadweaver_build",
	"tnef_build",
	"unitconversion_build",
	"wallet_build",
	"wayland_build",
	"widgetsaddons_build",
	"windowsystem_build",
	"xmlgui_build",
	"xmlrpcclient_build",
	"activities_run",
	"activities-stats_run",
	"activitymanagerd_run",
	"akonadi_run",
	"akonadicalendar_run",
	"akonadiconsole_run",
	"akonadicontacts_run",
	"akonadiimportwizard_run",
	"akonadimime_run",
	"akonadinotes_run",
	"akonadisearch_run",
	"akregator_run",
	"alarmcalendar_run",
	"apidox_run",
	"archive_run",
	"attica_run",
	"attica5_run",
	"auth_run",
	"baloo_run",
	"baloo-widgets_run",
	"baloo5_run",
	"bookmarks_run",
	"breeze_run",
	"breeze-gtk_run",
	"breeze-icons_run",
	"calendarcore_run",
	"calendarsupport_run",
	"calendarutils_run",
	"codecs_run",
	"completion_run",
	"config_run",
	"configwidgets_run",
	"contacts_run",
	"coreaddons_run",
	"crash_run",
	"dbusaddons_run",
	"decoration_run",
	"designerplugin_run",
	"discover_run",
	"dnssd_run",
	"doctools_run",
	"drkonqi_run",
	"ecm_run",
	"emoticons_run",
	"eventviews_run",
	"filemetadata_run",
	"frameworkintegration_run",
	"gapi_run",
	"globalaccel_run",
	"grantlee-editor_run",
	"grantleetheme_run",
	"gravatar_run",
	"guiaddons_run",
	"holidays_run",
	"hotkeys_run",
	"i18n_run",
	"iconthemes_run",
	"identitymanagement_run",
	"idletime_run",
	"imap_run",
	"incidenceeditor_run",
	"infocenter_run",
	"init_run",
	"itemmodels_run",
	"itemviews_run",
	"jobwidgets_run",
	"js_run",
	"jsembed_run",
	"kaddressbook_run",
	"kalarm_run",
	"kate_run",
	"kcmutils_run",
	"kdav_run",
	"kde-cli-tools_run",
	"kde-gtk-config_run",
	"kdeclarative_run",
	"kded_run",
	"kdelibs4support_run",
	"kdepim-addons_run",
	"kdepim-runtime5_run",
	"kdeplasma-addons_run",
	"kdesu_run",
	"kdewebkit_run",
	"kgamma5_run",
	"khtml_run",
	"kimageformats_run",
	"kio_run",
	"kirigami2_run",
	"kitinerary_run",
	"kmail_run",
	"kmail-account-wizard_run",
	"kmenuedit_run",
	"knotes_run",
	"kontact_run",
	"kontactinterface_run",
	"korganizer_run",
	"kosm_run",
	"kpkpass_run",
	"kpublictransport_run",
	"kquickcharts_run",
	"kross_run",
	"kscreen_run",
	"kscreenlocker_run",
	"ksmtp_run",
	"ksshaskpass_run",
	"ksysguard_run",
	"ksystemstats_run",
	"kwallet-pam_run",
	"kwayland-integration_run",
	"kwayland-server_run",
	"kwin_run",
	"kwrited_run",
	"layer-shell-qt_run",
	"ldap_run",
	"libkcddb_run",
	"libkcompactdisc_run",
	"libkdcraw_run",
	"libkdegames_run",
	"libkdepim_run",
	"libkeduvocdocument_run",
	"libkexiv2_run",
	"libkipi_run",
	"libkleo_run",
	"libksane_run",
	"libkscreen_run",
	"libksieve_run",
	"libksysguard_run",
	"mailcommon_run",
	"mailimporter_run",
	"mailtransport_run",
	"marble_run",
	"mbox_run",
	"mbox-importer_run",
	"mediaplayer_run",
	"messagelib_run",
	"milou_run",
	"mime_run",
	"newstuff_run",
	"notifications_run",
	"notifyconfig_run",
	"okular_run",
	"oxygen_run",
	"oxygen-icons5_run",
	"package_run",
	"parts_run",
	"people_run",
	"pim-data-exporter_run",
	"pimcommon_run",
	"pimtextedit_run",
	"plasma-browser-integration_run",
	"plasma-desktop_run",
	"plasma-disks_run",
	"plasma-framework_run",
	"plasma-integration_run",
	"plasma-pa_run",
	"plasma-sdk_run",
	"plasma-wayland-protocols_run",
	"plasma-workspace_run",
	"plasma-workspace-wallpapers_run",
	"plotting_run",
	"polkit-kde-agent-1_run",
	"powerdevil_run",
	"prison_run",
	"pty_run",
	"purpose_run",
	"qqc2-desktop-style_run",
	"runner_run",
	"service_run",
	"solid_run",
	"sonnet_run",
	"syndication_run",
	"syntaxhighlighting_run",
	"systemsettings_run",
	"texteditor_run",
	"textwidgets_run",
	"threadweaver_run",
	"tnef_run",
	"unitconversion_run",
	"wallet_run",
	"wayland_run",
	"widgetsaddons_run",
	"windowsystem_run",
	"xmlgui_run",
	"xmlrpcclient_run",
};
size_t use_kde_rel_len = nitems(use_kde_rel);
const char *use_qt_rel[] = {
	"3d",
	"assistant",
	"buildtools",
	"charts",
	"concurrent",
	"connectivity",
	"core",
	"datavis3d",
	"dbus",
	"declarative",
	"declarative-test",
	"designer",
	"diag",
	"doc",
	"examples",
	"gamepad",
	"graphicaleffects",
	"gui",
	"help",
	"imageformats",
	"l10n",
	"linguist",
	"linguisttools",
	"location",
	"multimedia",
	"network",
	"networkauth",
	"opengl",
	"paths",
	"phonon4",
	"pixeltool",
	"plugininfo",
	"printsupport",
	"qdbus",
	"qdbusviewer",
	"qdoc",
	"qdoc-data",
	"qev",
	"qmake",
	"quick3d",
	"quickcontrols",
	"quickcontrols2",
	"quicktimeline",
	"remoteobjects",
	"script",
	"scripttools",
	"scxml",
	"sensors",
	"serialbus",
	"serialport",
	"speech",
	"sql",
	"sql-ibase",
	"sql-mysql",
	"sql-odbc",
	"sql-pgsql",
	"sql-sqlite2",
	"sql-sqlite3",
	"sql-tds",
	"svg",
	"testlib",
	"uiplugin",
	"uitools",
	"virtualkeyboard",
	"wayland",
	"webchannel",
	"webengine",
	"webglplugin",
	"webkit",
	"websockets",
	"websockets-qml",
	"webview",
	"widgets",
	"x11extras",
	"xml",
	"xmlpatterns",
	"3d_build",
	"assistant_build",
	"buildtools_build",
	"charts_build",
	"concurrent_build",
	"connectivity_build",
	"core_build",
	"datavis3d_build",
	"dbus_build",
	"declarative_build",
	"declarative-test_build",
	"designer_build",
	"diag_build",
	"doc_build",
	"examples_build",
	"gamepad_build",
	"graphicaleffects_build",
	"gui_build",
	"help_build",
	"imageformats_build",
	"l10n_build",
	"linguist_build",
	"linguisttools_build",
	"location_build",
	"multimedia_build",
	"network_build",
	"networkauth_build",
	"opengl_build",
	"paths_build",
	"phonon4_build",
	"pixeltool_build",
	"plugininfo_build",
	"printsupport_build",
	"qdbus_build",
	"qdbusviewer_build",
	"qdoc_build",
	"qdoc-data_build",
	"qev_build",
	"qmake_build",
	"quick3d_build",
	"quickcontrols_build",
	"quickcontrols2_build",
	"quicktimeline_build",
	"remoteobjects_build",
	"script_build",
	"scripttools_build",
	"scxml_build",
	"sensors_build",
	"serialbus_build",
	"serialport_build",
	"speech_build",
	"sql_build",
	"sql-ibase_build",
	"sql-mysql_build",
	"sql-odbc_build",
	"sql-pgsql_build",
	"sql-sqlite2_build",
	"sql-sqlite3_build",
	"sql-tds_build",
	"svg_build",
	"testlib_build",
	"uiplugin_build",
	"uitools_build",
	"virtualkeyboard_build",
	"wayland_build",
	"webchannel_build",
	"webengine_build",
	"webglplugin_build",
	"webkit_build",
	"websockets_build",
	"websockets-qml_build",
	"webview_build",
	"widgets_build",
	"x11extras_build",
	"xml_build",
	"xmlpatterns_build",
	"3d_run",
	"assistant_run",
	"buildtools_run",
	"charts_run",
	"concurrent_run",
	"connectivity_run",
	"core_run",
	"datavis3d_run",
	"dbus_run",
	"declarative_run",
	"declarative-test_run",
	"designer_run",
	"diag_run",
	"doc_run",
	"examples_run",
	"gamepad_run",
	"graphicaleffects_run",
	"gui_run",
	"help_run",
	"imageformats_run",
	"l10n_run",
	"linguist_run",
	"linguisttools_run",
	"location_run",
	"multimedia_run",
	"network_run",
	"networkauth_run",
	"opengl_run",
	"paths_run",
	"phonon4_run",
	"pixeltool_run",
	"plugininfo_run",
	"printsupport_run",
	"qdbus_run",
	"qdbusviewer_run",
	"qdoc_run",
	"qdoc-data_run",
	"qev_run",
	"qmake_run",
	"quick3d_run",
	"quickcontrols_run",
	"quickcontrols2_run",
	"quicktimeline_run",
	"remoteobjects_run",
	"script_run",
	"scripttools_run",
	"scxml_run",
	"sensors_run",
	"serialbus_run",
	"serialport_run",
	"speech_run",
	"sql_run",
	"sql-ibase_run",
	"sql-mysql_run",
	"sql-odbc_run",
	"sql-pgsql_run",
	"sql-sqlite2_run",
	"sql-sqlite3_run",
	"sql-tds_run",
	"svg_run",
	"testlib_run",
	"uiplugin_run",
	"uitools_run",
	"virtualkeyboard_run",
	"wayland_run",
	"webchannel_run",
	"webengine_run",
	"webglplugin_run",
	"webkit_run",
	"websockets_run",
	"websockets-qml_run",
	"webview_run",
	"widgets_run",
	"x11extras_run",
	"xml_run",
	"xmlpatterns_run",
};
size_t use_qt_rel_len = nitems(use_qt_rel);
const char *use_pyqt_rel[] = {
	"chart",
	"networkauth",
	"pyqt5",
	"pysip",
	"qscintilla2",
	"qtbuilder",
	"sip",
	"webengine",
	"chart_build",
	"networkauth_build",
	"pyqt5_build",
	"pysip_build",
	"qscintilla2_build",
	"qtbuilder_build",
	"sip_build",
	"webengine_build",
	"chart_run",
	"networkauth_run",
	"pyqt5_run",
	"pysip_run",
	"qscintilla2_run",
	"qtbuilder_run",
	"sip_run",
	"webengine_run",
	"chart_test",
	"networkauth_test",
	"pyqt5_test",
	"pysip_test",
	"qscintilla2_test",
	"qtbuilder_test",
	"sip_test",
	"webengine_test",
};
size_t use_pyqt_rel_len = nitems(use_pyqt_rel);
const char *static_shebang_langs[] = {
	"bash",
	"java",
	"ksh",
	"lua",
	"perl",
	"php",
	"python",
	"ruby",
	"tcl",
	"tk",
};
size_t static_shebang_langs_len = nitems(static_shebang_langs);
