${PORTEDIT} bump-revision $input | diff -L $expected -L $actual -u $expected -
<<<<<<<<<
PORTNAME=	lepton-eda
PORTVERSION=	1.9.7
PORTREVISION?=	2
DISTVERSIONSUFFIX=	-20181211
CATEGORIES=	cad
<<<<<<<<<
PORTNAME=	lepton-eda
PORTVERSION=	1.9.7
DISTVERSIONSUFFIX=	-20181211
PORTREVISION?=	3
CATEGORIES=	cad
