/*-
 * SPDX-License-Identifier: BSD-2-Clause-FreeBSD
 *
 * Copyright (c) 2021 Tobias Kortkamp <tobik@FreeBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#pragma once

struct LineIterator;
struct Mempool;

bool can_use_colors(FILE *);

char *symlink_read(int, const char *, struct Mempool *);
bool symlink_update(int, const char *, const char *, struct Mempool *, char **);

struct LineIterator *line_iterator(FILE *, ssize_t, ssize_t);
void line_iterator_cleanup(struct LineIterator **);
bool line_iterator_next(struct LineIterator **, size_t *, char **, size_t *);
bool mkdirp(const char *);
char *slurp(FILE *, struct Mempool *);

#define LINE_FOREACH_SLICE(FILE, A, B, VAR) \
	ITERATOR_FOREACH(Line, line, char *, char **, VAR, NULL, size_t, size_t *, VAR##_len, 0, FILE, A, B)
#define LINE_FOREACH(FILE, VAR) \
	LINE_FOREACH_SLICE(FILE, 0, -1, VAR)
