USES=		compiler:c++11-lang iconv compiler:c++14-lang

FOO_USES_OFF=	python:3.5+,build python
FOO_USES=	python:3.5+,build python
<<<<<<<<<
USES=		compiler:c++11-lang iconv

FOO_USES_OFF=	python:3.5+,build
FOO_USES=	python:3.5+,build
