${PORTEDIT} set-version 2.4.0-Matrix $input | diff -L $expected -L $actual -u $expected -
<<<<<<<<<
DISTVERSION=	2.3.0
DISTVERSIONSUFFIX=	-Matrix
<<<<<<<<<
DISTVERSION=	2.4.0
DISTVERSIONSUFFIX=	-Matrix
