PORTSCOUT=	limit:^[0-9.]+$$ # bla
PORTSCOUT=	limit:^[0-9.]+$$	# bla
<<<<<<<<<
VARIABLE :line 1 :name "PORTSCOUT" :modifier = :comment "# bla" :words [1]["limit:^[0-9.]+$$"]
VARIABLE :line 2 :name "PORTSCOUT" :modifier = :comment "# bla" :words [1]["limit:^[0-9.]+$$"]
