/*-
 * SPDX-License-Identifier: BSD-2-Clause-FreeBSD
 *
 * Copyright (c) 2019 Tobias Kortkamp <tobik@FreeBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "config.h"

#include <stdbool.h>
#include <stdio.h>
#include <string.h>

#include <libias/mempool.h>
#include <libias/str.h>

#include "ast.h"
#include "variable.h"

bool
parse_variable(struct Mempool *extpool, const char *buf, char **name, enum ASTVariableModifier *mod)
{
	SCOPE_MEMPOOL(pool);

	size_t len = strlen(buf);

	if (len < 2) {
		return false;
	}
	if (buf[len - 1] != '=') {
		return false;
	}

	*mod = AST_VARIABLE_MODIFIER_ASSIGN;
	size_t i = 2;
	switch (buf[len - 2]) {
	case ':':
		*mod = AST_VARIABLE_MODIFIER_EXPAND;
		break;
	case '!':
		*mod = AST_VARIABLE_MODIFIER_SHELL;
		break;
	case '?':
		*mod = AST_VARIABLE_MODIFIER_OPTIONAL;
		break;
	case '+':
		*mod = AST_VARIABLE_MODIFIER_APPEND;
		break;
	default:
		i = 1;
		break;
	}

	*name = str_trimr(pool, str_ndup(pool, buf, strlen(buf) - i));
	if (strcmp(*name, "") == 0) {
		*name = NULL;
		return false;
	} else {
		mempool_move(pool, *name, extpool);
		return true;
	}
}
