# From astro/geographiclib/Makefile
MASTER_SITES=	SF/geographiclib/distrib\
		SF/geographiclib:geoids,gravity,magnetic
<<<<<<<<<
# From astro/geographiclib/Makefile
MASTER_SITES=	SF/geographiclib/distrib \
		SF/geographiclib:geoids,gravity,magnetic
