ALL_TARGET=	c b a
INSTALL_TARGET=	install.bin install.lib install.data install.hdr \
		install.staticlib
TEST_TARGET=	x y z
<<<<<<<<<
ALL_TARGET=	c b a
INSTALL_TARGET=	install.bin install.lib install.data install.hdr \
		install.staticlib
TEST_TARGET=	x y z
