LIB_DEPENDS=	libsndio.so:audio/sndio libcurl.so:ftp/curl \
	libABC.so:asfd/asdfa \
	libX11.so:x11/xlib
<<<<<<<<<
LIB_DEPENDS=	libABC.so:asfd/asdfa \
		libcurl.so:ftp/curl \
		libsndio.so:audio/sndio \
		libX11.so:x11/xlib
