${PORTEDIT} set-version 0.1.1-166-g7a3765a $input | diff -L $expected -L $actual -u $expected -
<<<<<<<<<
DISTVERSIONPREFIX=	v
DISTVERSION=	0.9.2-259
DISTVERSIONSUFFIX=	-gc07c115
<<<<<<<<<
DISTVERSION=	0.1.1-166
DISTVERSIONSUFFIX=	-g7a3765a
