${PORTEDIT} bump-epoch $input | diff -L $expected -L $actual -u $expected -
<<<<<<<<<
PORTNAME=	sndio
PORTVERSION=	1.5.0
PORTREVISION=	1
PORTEPOCH=	1
CATEGORIES=	audio
MASTER_SITES=	http://www.sndio.org/
<<<<<<<<<
PORTNAME=	sndio
PORTVERSION=	1.5.0
PORTEPOCH=	2
CATEGORIES=	audio
MASTER_SITES=	http://www.sndio.org/
