PLIST_FILES=	bin/asasdf "@sample etc/zzaf.sample" sbin/bla

HELLO_PLIST_FILES=	bin/asa "@sample etc/muh" sbin/bla
<<<<<<<<<
PLIST_FILES=	bin/asasdf \
		"@sample etc/zzaf.sample" \
		sbin/bla

HELLO_PLIST_FILES=	bin/asa \
			"@sample etc/muh" \
			sbin/bla
