MAKE_ENV+=     CXXFLAGS_WARN="${CXXFLAGS_WARN}"
MAKE_ENV+=     GNUSED=${LOCALBASE}/bin/gsed
MAKE_ENV+=     DISPLAY=
MAKE_ENV+=     verbose=1
<<<<<<<<<
VARIABLE :line 1 :name "MAKE_ENV" :modifier += :words [1]["CXXFLAGS_WARN="${CXXFLAGS_WARN}""]
VARIABLE :line 2 :name "MAKE_ENV" :modifier += :words [1]["GNUSED=${LOCALBASE}/bin/gsed"]
VARIABLE :line 3 :name "MAKE_ENV" :modifier += :words [1]["DISPLAY="]
VARIABLE :line 4 :name "MAKE_ENV" :modifier += :words [1]["verbose=1"]
