CONTENTS=	'NAME=FreeBSD\0'$\
		'VERSION=${VERSION}\0'$\
		'VERSION_ID=${VERSION_ID}\0'$\
		'ID=freebsd\0'$\
		'ANSI_COLIR="0;31"\0'$\
		'PRETTY_NAME="FreeBSD ${VERSION}"\0'$\
		'CPE_NAME=cpe:/o:freebsd:freebsd:${VERSION_ID}\0'$\
		'HOME_URL=https://freebsd.org/\0'$\
		'BUG_REPORT_URL=https://bugs.freebsd.org'
<<<<<<<<<
CONTENTS=	'NAME=FreeBSD\0'$\
		'VERSION=${VERSION}\0'$\
		'VERSION_ID=${VERSION_ID}\0'$\
		'ID=freebsd\0'$\
		'ANSI_COLIR="0;31"\0'$\
		'PRETTY_NAME="FreeBSD ${VERSION}"\0'$\
		'CPE_NAME=cpe:/o:freebsd:freebsd:${VERSION_ID}\0'$\
		'HOME_URL=https://freebsd.org/\0'$\
		'BUG_REPORT_URL=https://bugs.freebsd.org'
