USERS=		_sndio
GROUPS=		_sndio

# Parallel build leads to problems, but sndio is very quick to compile
# as is so not worth fixing
MAKE_JOBS_UNSAFE=	yes
<<<<<<<<<
USERS=		_sndio
GROUPS=		_sndio

# Parallel build leads to problems, but sndio is very quick to compile
# as is so not worth fixing
MAKE_JOBS_UNSAFE=	yes
