DOS2UNIX_FILES=	third_party/skia/third_party/vulkanmemoryallocator/include/vk_mem_alloc.h
FIREFOX_ICON_SRC=	${PREFIX}/lib/${MOZILLA}/browser/chrome/icons/default/default48.png
FIREFOX_DESKTOP=	${MOZSRC}/taskcluster/docker/${MOZILLA}-snap/${MOZILLA}.desktop
CUSTOM_VARIABLE=	this_is_a_very_long_token_that_should_appear_twice_intentionally_but_not_really \
			this_is_a_very_long_token_that_should_appear_twice_intentionally
<<<<<<<<<
DOS2UNIX_FILES=		third_party/skia/third_party/vulkanmemoryallocator/include/vk_mem_alloc.h
FIREFOX_ICON_SRC=	${PREFIX}/lib/${MOZILLA}/browser/chrome/icons/default/default48.png
FIREFOX_DESKTOP=	${MOZSRC}/taskcluster/docker/${MOZILLA}-snap/${MOZILLA}.desktop
CUSTOM_VARIABLE=	this_is_a_very_long_token_that_should_appear_twice_intentionally_but_not_really \
			this_is_a_very_long_token_that_should_appear_twice_intentionally
