LDFLAGS+=	-fstack-protector-strong
LDFLAGS+=	-Wl,-E # plugins
LDFLAGS+=	-Wl,--as-needed # ICE, SM, Xext
<<<<<<<<<
LDFLAGS+=	-fstack-protector-strong
LDFLAGS+=	-Wl,-E # plugins
LDFLAGS+=	-Wl,--as-needed # ICE, SM, Xext
