PORTNAME=	tokei
DISTVERSIONPREFIX=	v
DISTVERSION=	9.0.0
CATEGORIES=	devel

LICENSE=	APACHE20 MIT
LICENSE_COMB=	dual
LICENSE_FILE_APACHE20=	${WRKSRC}/LICENCE-APACHE
LICENSE_FILE_MIT=	${WRKSRC}/LICENCE-MIT
<<<<<<<<<
PORTNAME=	tokei
DISTVERSIONPREFIX=	v
DISTVERSION=	9.0.0
CATEGORIES=	devel

LICENSE=	APACHE20 MIT
LICENSE_COMB=	dual
LICENSE_FILE_APACHE20=	${WRKSRC}/LICENCE-APACHE
LICENSE_FILE_MIT=	${WRKSRC}/LICENCE-MIT
