OPENMPI2_VARS=		cc=${LOCALBASE}/mpi/openmpi2/bin/mpicc \
			cxx=${LOCALBASE}/mpi/openmpi2/bin/mpicxx
<<<<<<<<<
OPENMPI2_VARS=	CC=${LOCALBASE}/mpi/openmpi2/bin/mpicc \
		CXX=${LOCALBASE}/mpi/openmpi2/bin/mpicxx
