.for binfile in envdir envuidgid fghack multilog pgrphack readproctitle setlock setuidgid softlimit supervise svc svok svscan svstat svup tai64n tai64nlocal
.endfor
.for docfile in README LICENSE CHANGES CHANGES.djb
.endfor
.for manfile in ${MANFILES}
.endfor
<<<<<<<<<
.for binfile in envdir envuidgid fghack multilog pgrphack readproctitle setlock \
	setuidgid softlimit supervise svc svok svscan svstat svup tai64n \
	tai64nlocal
.endfor
.for docfile in README LICENSE CHANGES CHANGES.djb
.endfor
.for manfile in ${MANFILES}
.endfor
