TEX_FORMAT_MLTEX?= \
	"mllatex pdftex language.dat -translate-file=cp227.tcx -mltex mllatex.ini" \
	"mltex pdftex - -translate-file=cp227.tcx -mltex mltex.ini"
<<<<<<<<<
TEX_FORMAT_MLTEX?=	"mllatex pdftex language.dat -translate-file=cp227.tcx -mltex mllatex.ini" \
			"mltex pdftex - -translate-file=cp227.tcx -mltex mltex.ini"
