${PORTEDIT} bump-revision $input | diff -L $expected -L $actual -u $expected -
<<<<<<<<<
DISTVERSION?=	12.8
# PORTREVISION must be ?= otherwise, all slave ports get this PORTREVISION and
# not their own.  Probably best to keep it at ?=0 when reset here too.
PORTREVISION?=	0
<<<<<<<<<
DISTVERSION?=	12.8
# PORTREVISION must be ?= otherwise, all slave ports get this PORTREVISION and
# not their own.  Probably best to keep it at ?=0 when reset here too.
PORTREVISION?=	1
