${PORTEDIT} set-version 0.61-335-gdf58b85 $input | diff -L $expected -L $actual -u $expected -
<<<<<<<<<
# $FreeBSD$

post-patch:
<<<<<<<<<
# $FreeBSD$

DISTVERSION=	0.61-335
DISTVERSIONSUFFIX=	-gdf58b85

post-patch:
