.if defined(DESTDIR) && !empty(DESTDIR) && !defined(CHROOTED) && \
	!defined(BEFOREPORTMK) && !defined(INOPTIONSMK) && \
	!defined(BEFOREPORTMK) && !defined(INOPTIONSMK)
_FOOBAR=	lasjflas
.endif
<<<<<<<<<
.if defined(DESTDIR) && !empty(DESTDIR) && !defined(CHROOTED) && \
	!defined(BEFOREPORTMK) && !defined(INOPTIONSMK) && \
	!defined(BEFOREPORTMK) && !defined(INOPTIONSMK)
_FOOBAR=	lasjflas
.endif
