${PORTEDIT} set-version longer-prefix-0.9.2-259-gc07c115 $input | diff -L $expected -L $actual -u $expected -
<<<<<<<<<
PORTNAME=	sndio
DISTVERSION=	0.0.1
PORTREVISION=	0
<<<<<<<<<
PORTNAME=	sndio
DISTVERSIONPREFIX=	longer-prefix-
DISTVERSION=	0.9.2-259
DISTVERSIONSUFFIX=	-gc07c115
PORTREVISION=	0
