${PORTEDIT} bump-revision $input | diff -L $expected -L $actual -u $expected -
<<<<<<<<<
PORTNAME=	ghostwriter
DISTVERSIONPREFIX=	v
DISTVERSION=	1.7.4
PORTREVISION?=	1
CATEGORIES=	editors
<<<<<<<<<
PORTNAME=	ghostwriter
DISTVERSIONPREFIX=	v
DISTVERSION=	1.7.4
PORTREVISION?=	2
CATEGORIES=	editors
