${PORTEDIT} set-version v0.9.2 $input | diff -L $expected -L $actual -u $expected -
<<<<<<<<<
DISTVERSIONPREFIX=	v
DISTVERSION=	0.9.2-259
DISTVERSIONSUFFIX=	-gc07c115
<<<<<<<<<
DISTVERSIONPREFIX=	v
DISTVERSION=	0.9.2
