OPTIONS_GROUP=		SOUND
OPTIONS_GROUP_SOUND=	SNDIO
OPTIONS_GROUP_VIDEO=	OPENGL

.include <bsd.port.mk>
<<<<<<<<<
# Options definitions
OPTIONS_GROUP
OPTIONS_GROUP_SOUND

# Unknown variables
# WARNING:
# The following variables were not recognized.
# They could just be typos or Portclippy needs to be made aware of them.
# Please double check them.
#
# Prefix them with an _ or wrap in '.ifnmake portclippy' to tell
# Portclippy to ignore them.
#
# If in doubt please report this on portfmt's bug tracker:
# https://github.com/t6/portfmt/issues
OPTIONS_GROUP_VIDEO
