${PORTEDIT} set-version v0.9.2-259-gc07c115 $input | diff -L $expected -L $actual -u $expected -
<<<<<<<<<
PORTVERSION=	0.9.2
<<<<<<<<<
DISTVERSIONPREFIX=	v
DISTVERSION=	0.9.2-259
DISTVERSIONSUFFIX=	-gc07c115
