A=foo\ bar
B=foo bar
<<<<<<<<<
variable-start              1 A= -
variable-token              1 A= foo\ bar
variable-end                1 A= -
variable-start              2 B= -
variable-token              2 B= foo
variable-token              2 B= bar
variable-end                2 B= -
