printf "CARGO_CRATES=" | ${PORTEDIT} merge $input | diff -L $expected -L $actual -u $expected -
<<<<<<<<<
PORTNAME=

# enable all output serialization formats
CARGO_FEATURES=	all
<<<<<<<<<
PORTNAME=

CARGO_CRATES=

# enable all output serialization formats
CARGO_FEATURES=	all
