// SPDX-License-Identifier: BSD-2-Clause-FreeBSD
//
// Copyright (c) 2021 Tobias Kortkamp <tobik@FreeBSD.org>
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
#pragma once

struct Distinfo;
struct Array;
struct Mempool;

#define DISTINFO_MAX_DIGEST_LEN 64

struct DistinfoEntry {
	const char *filename;
	uint8_t digest[DISTINFO_MAX_DIGEST_LEN];
	size_t digest_len;
	off_t size;
};

struct Distinfo *distinfo_parse(FILE *, struct Mempool *, struct Array **);
struct Distinfo *distinfo_new(void);
void distinfo_free(struct Distinfo *);
struct Array *distinfo_entries(struct Distinfo *, struct Mempool *);
struct DistinfoEntry *distinfo_entry(struct Distinfo *, const char *);
void distinfo_add_entry(struct Distinfo *, struct DistinfoEntry *);
void distinfo_remove_entry(struct Distinfo *, const char *);
time_t distinfo_timestamp(struct Distinfo *);
void distinfo_set_timestamp(struct Distinfo *, time_t);
void distinfo_entry_serialize(struct DistinfoEntry *, FILE *);
void distinfo_serialize(struct Distinfo *, FILE *);
